/* eslint-disable @typescript-eslint/no-empty-interface */
import { SchemaInterface } from './SchemaInterface'
import axios, { AxiosRequestConfig, AxiosResponse, Method } from 'axios'
import stubResolver from '@/stubs/StubResolver'
import domainResolver from '@/api/DomainResolver'

export abstract class Handler {
  abstract handle: any
}

export interface RequestInterface {
  method: Method;
  url: string;
  data: Record<string, any> | null;
  headers: Record<string, any> | null;
}

interface RequestHandleFunction {
  (request: RequestInterface): void;
}

export abstract class RequestHandler extends Handler {
  abstract handle: RequestHandleFunction
}

export interface ResponseInterface extends AxiosResponse {

}

interface ResponseHandleFunction {
  (response: ResponseInterface): void;
}

export abstract class ResponseHandler extends Handler {
  abstract handle: ResponseHandleFunction
}

export class HttpClient {
  private baseUrl: string
  private handlers: Handler[] = []

  constructor (baseUrl: string) {
    this.baseUrl = baseUrl
  }

  async syncBaseUrlWithStand () {
    await domainResolver
      .resolve()
      .then((url) => {
        this.baseUrl = url || this.baseUrl
      })
  }

  send (schema: SchemaInterface, data: any | null = null): Promise<AxiosResponse> {
    this.syncBaseUrlWithStand()

    let url = schema.url
    const preparedData = data
    for (const key in data) {
      if (url.indexOf('{' + key + '}') > -1) {
        url = url.replace('{' + key + '}', preparedData[key])
        delete preparedData[key]
      }
    }
    console.log(data, 'data')
    const requestData = {
      method: schema.method as Method,
      url: this.baseUrl + url,
      data: preparedData,
      headers: {
        'Content-Type': 'application/json;charset=utf-8'
      }
    } as RequestInterface

    /**
     * todo перевести на промисы +
     */
    this.handlers
      .filter((handler: Handler) => {
        return handler instanceof RequestHandler
      })
      .map((handler: Handler) => {
        handler.handle(requestData)
        return handler
      })

    const stub = stubResolver.handle(schema.code)
    const axiosData = requestData as AxiosRequestConfig

    if (schema.prefix) {
      const temp: any = {}
      temp[schema.prefix] = axiosData.data
      axiosData.data = temp
    }

    if (axiosData.method === 'GET') {
      axiosData.params = axiosData.data
      delete axiosData.data
    }
    const response = stub || axios(axiosData)

    return response
      .then((response: any) => {
        /**
         * todo перевести на промисы +
         */
        this.handlers
          .filter((handler: Handler) => {
            return handler instanceof ResponseHandler
          })
          .map((handler: Handler) => {
            handler.handle(response)
            return handler
          })

        return response
      })
  }

  addHandler (handler: Handler) {
    this.handlers.push(handler)
    return this
  }
}
